"""Use compclust from the web

TODO:
  be able to save datasources for later loading
  be able to browse saved datasources and load them

  xml-rpc/functional interface to add/remove stuff
TODO:
"""
import operator
import os
import math
import re
import shelve
import string
import StringIO
import sys
import thread
import time
import traceback
import types
import urllib
#import warnings
#from warnings import warn

# make sure we have a home
if not os.environ.has_key('HOME'):
  os.environ['HOME'] = '/tmp'

#try:
#  from mod_python import apache
#  from mod_python import util
#  warnfile = "/tmp/ccw.log"
#except:
#  from localmod_python import apache
#  from localmod_python import util
#  warnfile = sys.stderr
#
from quixote.errors import *
from quixote.html import htmlescape
from quixote import get_session_manager

from simpletal import simpleTAL, simpleTALES, simpleTALUtils
talCache = simpleTALUtils.TemplateCache()

from compClust.mlx import views
from compClust.mlx.ML_Algorithm import ML_Composable_Algorithm
from compClust.util import DistanceMetrics
from compClust.util import WrapperPlugins
from compClust.util import WrapperParameters
from compClust.util.WrapperParameters import HTMLFormatParameter
from compClust.util.WrapperBuilder  import WrapperBuilder

from compClust.iplot import IPlotAgg as IPlot
from compClust.iplot.PlotPage import PlotPage

from compClust.gui.PlotCache import PlotCache
from compClust.gui.DataSource import DataSource
from compClust.gui.DataManager import DataManager
from compClust.gui.LabelingSource import LabelingSource

import compClust.gui.DataManager
# Setup warning manager
#class DebugWarning(Warning): pass
#class InfoWarning(Warning): pass
#
#def showwarning(message, category, filename, linenumber, f=warnfile):
#  """Write warnings to our favorite location
#  """
#  if type(f) in types.StringTypes:
#    f = open(f, 'a')
#    
#  timestamp = time.strftime("%Y %b %d %H:%M:%S ")
#  warn = warnings.formatwarning(message, category, filename, linenumber)
#  warn = string.split(warn, os.linesep)[0]
#  f.write(string.join([timestamp, warn, os.linesep]))
#warnings.showwarning = showwarning
#warn("import complete", DebugWarning)

# ROOT_URL is the the base URI to our application
ROOT_URL = "/compclust/"

def _path_join(*args):
  """Abstract out the os.path.join as the '/' as path seperator is a unix-ism
  """
  return os.path.join(*args)

from quixote.util import StaticFile
templates = os.path.split(compClust.gui.DataManager.__file__)[0]
# when launching from within an ipython shell __file__ is relative
# instead of absolute
if not os.path.isabs(templates):
  templates = os.path.join(os.getcwd(), templates)
compclust_css = StaticFile(os.path.join(templates, "compclust.css"), mime_type="text/css")


_q_exports = ['_q_index', 'image', 'compclust', 'compclust_css']
#              'editdatasource', 'editlabelingsource', 'labelplot', 'cluster_detail_plot', 'cluster_trajectories', 'confusion_matrix_plot', 'roc', 'compclust_css', 'diane']

class NotLoggedInError(PublishError):
  def __init__(self, description, source_url):
    PublishError.__init__(self, description)
    self.source_url = source_url

def setXHTMLtype(req):
  if 'application/xhtml+xml' in req.get_accepted_types():
    req.response.set_content_type('application/xhtml+xml')
  else:
    req.response.set_content_type('text/html')

def template_header():
  return '''<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
<head>
  <title>Comp Clust</title>
  <link rel="stylesheet" href="%(compclust_css)s" type="text/css" />
</head>
<body>''' % {"compclust_css": _path_join(ROOT_URL, 'compclust_css')}

def template_footer():
  return '</html>'

def get_context(req, datasource=None):
  context = simpleTALES.Context()
  context.addGlobal('menu', compclust.menu(req, datasource))
  context.addGlobal('compclust_css', _path_join(ROOT_URL, 'compclust_css'))
  return context

def get_template(req, filename, context=None):
  if context is None:
    context = simpleTALES.Context()
    
  page = StringIO.StringIO()
  setXHTMLtype(req)
  template= talCache.getXMLTemplate(os.path.join(templates,filename+'.xml'))
  template.expand(context, page)
  return page.getvalue()

CLUSTERING_MISSING = 0
CLUSTERING_RUNNING = 1
CLUSTERING_FINISHED = 2
CLUSTERING_FAILED = 3

clustering_status_lock = thread.allocate()
clustering_status = {}

def clustering_thread(name, wrapper_builder, datasource):
  # FIXME: perhaps we should track which jobs a created and delete them from
  # a queue when done
  threadid = str(thread.get_ident())
  clustering_status_lock.acquire()
  clustering_status[threadid] = CLUSTERING_RUNNING
  clustering_status_lock.release()
  print "starting run", name
  try:
    wrapper_builder.run()
    labeling = wrapper_builder.getLabeling()
    if labeling is None:
      print "error running clustering"
      clustering_status_lock.acquire()
      clustering_status[threadid] = CLUSTERING_FAILED
      clustering_status_lock.release()
    else:
      labeling.setName(name)
      print "finished run", name
      source = LabelingSource(name, isrow=True, isannotation=False)
      clustering_status_lock.acquire()
      datasource._labeling_sources[name] = source
      clustering_status[threadid] = CLUSTERING_FINISHED
      clustering_status_lock.release()
  except Exception, e:
    clustering_status_lock.acquire()
    clustering_status[threadid] = CLUSTERING_FAILED
    clustering_status_lock.release()
    print traceback.print_exc()

wrapper_plugins = WrapperPlugins.WrapperPlugins()
  
    
class ClusteringUI:
  """UI for a particular clustering algorithm
  """
  _q_exports = ['action', 'status', 'subalgorithm']
  def __init__(self, datamanagerui, datasource, baseurl, wrapper_builder):
    self.datamanagerui = datamanagerui
    self.datasource = datasource
    self.wrapper_builder = wrapper_builder
    self.baseurl = os.path.join(baseurl, self.wrapper_builder[-1].__class__.__name__)
    
  def nameByClusteringParameters(self, parameters):
    """Create a name based on clustering parameters
    """
    labeling_parameters = ''
    for prop in parameters.getProperties().values():
      if prop.priority == WrapperParameters.Priority.REQUIRED:
        labeling_parameters+= ' %s=%s' % (prop.name[:3], parameters[prop.name])
    return labeling_parameters

  def action(self, req):
    """Execute results of Clustering Form submission
    """
    #wrapper_builder = self.__subalgorithm_list_parse(req)
    # get & validate variables
    for i in xrange(len(self.wrapper_builder)):
      algorithm = self.wrapper_builder[i]
      for name, prop in algorithm.parameters.getProperties().items():
        variable_name = self.__format_name(name, i)
        value = req.get_form_var(variable_name, algorithm.parameters[name])
        # Convert human lists into python lists for ListProperties
        if isinstance(prop, WrapperParameters.ListProperty):
          value = value.split()
        try:
          algorithm.parameters[name] = value
        except ValueError, e:
          raise QueryError(str(e))
      
    # come up with our name
    clustering_name = req.get_form_var('clusteringname',
                                       self.wrapper_builder[0].__class__.__name__)
    append_parameters = req.get_form_var('appendparameters', False)
    if append_parameters is not False:
      clustering_name += self.nameByClusteringParameters(self.wrapper_builder[0].parameters)
    # is there a labeling with that name?
    if not self.datasource.has_labeling_by_name(clustering_name):
      raise QueryError("Clusterings must be unique")
    
    # make dictionary for clustering algorithm
    if not self.wrapper_builder.validate():
      raise QueryError("Parameters failed validation")

    # run algorithm
    threadid = thread.start_new_thread(clustering_thread, (clustering_name, self.wrapper_builder, self.datasource))
    return req.redirect(os.path.join(self.baseurl, 'status?jobid=%s' % (threadid)))

  def status(self, req):
    threadid = req.get_form_var('jobid', None)
    
    # tell the user something happened
    html = [template_header()]
    html += [self.datamanagerui.menu(req, self.datasource)]
    html += ['<div id="compclustBody">']

    try:
      clustering_status_lock.acquire()
      if threadid is None or not clustering_status.has_key(threadid):
        html += ["<p>bad job identifier</p>"]
      elif clustering_status[threadid] == CLUSTERING_RUNNING:
        html += ["<p>running job %s</p>" % (threadid)]
        html += ["<p>This page will automatically reload every 15 seconds to check to see if the job is done.</p>"]
        req.response.set_header('refresh', '15')
      elif clustering_status[threadid] == CLUSTERING_FINISHED:
        html += ["<p>job %s finished successfully</p>" % (threadid)]
      elif clustering_status[threadid] == CLUSTERING_FAILED:
        html += ["<p>job %s finished successfully</p>" % (threadid)]
    finally:
      clustering_status_lock.release()
    
    html += ['</div>']
    html += [template_footer()]
    
    return string.join(html, os.linesep)

  def __format_name(self, name, id=None):
    """Format a parameter name in a way that seperates all the various
    parameters for each algorithm in a wrapper_builder chain.
    """
    if id is None:
      return name
    else:
      return name+"."+str(id)
  
  def __format_parameters(self):
    """
    Construct all the gory xHTML necessary to prompt for the required
    and optional clustering algorithm paramters for each algorithm
    that was added to the wrapper builder.
    """
    algorithms_html = []
    for i in xrange(len(self.wrapper_builder)):
      algorithm = self.wrapper_builder[i]
      required = []
      optional = []
      for name, prop in algorithm.parameters.getProperties().items():
        munged_name = self.__format_name(name, i)
        value = algorithm.parameters[name]
        if prop.priority == WrapperParameters.Priority.REQUIRED:
          required += HTMLFormatParameter(name, prop, value, munged_name)
        elif prop.priority == WrapperParameters.Priority.OPTIONAL:
          optional += HTMLFormatParameter(name, prop, value, munged_name)
      algorithms_html.append({'clusteringname': algorithm.__class__.__name__,
                              'required': required,
                              'optional': optional})
    return algorithms_html

  def _q_lookup(self, req, name):
    self.wrapper_builder.append(wrapper_plugins[name])
    return ClusteringUI(self.datamanagerui, self.datasource, self.baseurl, wrapper_plugins, self.wrapper_builder)
  
  def _q_index(self, req):
    context = get_context(req, self.datasource)
    # prompt for parameters
    context.addGlobal("algorithms_parameters", self.__format_parameters())
    # submit to the action function which runs the algorithm
    context.addGlobal('clusteringname', self.wrapper_builder[0].__class__.__name__)
    return get_template(req, 'Clustering', context)

class ClusteringManagerUI:
  """Allow browsing clustering algorithms that are installed.
  """
  _q_exports = []
  def __init__(self, datamanagerui, datasource, baseurl, wrapper_builder=None):
    self.datamanagerui = datamanagerui
    self.datasource = datasource
    if wrapper_builder is None:
      self.baseurl = os.path.join(baseurl, 'cluster')
      self.wrapper_builder = WrapperBuilder(self.datasource.dataset)
    else:
      self.baseurl = os.path.join(baseurl, wrapper_builder[-1].__class__.__name__)
      self.wrapper_builder = wrapper_builder

  def _q_lookup(self, req, name):
    self.wrapper_builder.append(wrapper_plugins[name])
    print self.wrapper_builder.algorithm_names()
    if self.wrapper_builder.isFinalized():
      return ClusteringUI(self.datamanagerui, self.datasource, self.baseurl, self.wrapper_builder)
    else:
      return ClusteringManagerUI(self.datamanagerui, self.datasource, self.baseurl, self.wrapper_builder)
  
  def _q_index(self, req):
    context = get_context(req, self.datasource)
    subalgorithm_names = []
    subalgorithm_names.append(string.join(self.wrapper_builder.algorithm_names,
                                          " : "))
    subalgorithms = []
    for alg_name in wrapper_plugins.keys():
      subalgorithms.append((os.path.join(self.baseurl, alg_name), alg_name))
    context.addGlobal('subalgorithms', subalgorithms)
    # submit back to the index page
    context.addGlobal('action', '')
    context.addGlobal('clusteringname', self.wrapper_builder[0].__class__.__name__)
    return get_template(req, 'ClusteringBuilder', context)

  def _q_index(self, req):
    context = get_context(req, self.datasource)
    # build list of names of algorithms in the wrapper builder
    if len(self.wrapper_builder) > 0:
      context.addGlobal('subalgorithm_names',
                     string.join(self.wrapper_builder.algorithm_names()," : "))
    # build list of algorithms that can be added to the chain
    algorithms = []
    for name in wrapper_plugins.keys():
      if issubclass(wrapper_plugins[name], ML_Composable_Algorithm):
        composable = 'Yes'
      else:
        composable = 'No'
      algorithms.append({'name': name,
                         'href': os.path.join(self.baseurl, name),
                         'composable':  composable})
    context.addGlobal("algorithms", algorithms)      
    return get_template(req, 'ClusteringManager', context)
  
class PlotUI:
  _q_exports = ['download', 'sendplot']
  supported_formats={'BMP' : 'image/x-ms-bmp',
                     'GIF' : 'image/gif',
                     'JPEG': 'image/jpeg',
                     #'PALM': 'application/vnd.palm',
                     #'PCX' : 'image/pcx',
                     'PDF' : 'application/pdf',
                     'EPS' : 'application/postscript',
                     #'PS'  : 'application/postscript'
                     'PNG' : 'image/png',
                     }
  sorted_formats = supported_formats.keys()
  sorted_formats.sort()
  vector_formats = ['PS', 'EPS', 'SVG']
  def __init__(self, datasource):
    self.datasource = datasource
    
  def _q_lookup(self, request, name):
    print "plotui_lookup", len(self.datasource), id(self.datasource)
    request.response.cache = 3600
    p = self.datasource.get_plot_by_id(name)
    if p is not None:
      request.response.set_content_type('image/png')
      plot = open(p.image_filename, 'rb').read()
      return plot
    else:
      request.response.set_status(404)
      return ""

  def download(self, req):
    """Prompt for download settings
    """
    plotid=req.get_form_var('plotid',None)
    if plotid is None:
      raise TraversalError("invalid url")
    p = self.datasource.get_plot_by_id(plotid)
    context = get_context(req, self.datasource)
    context.addGlobal('plotid', p.id)
    context.addGlobal('format',html_combo_selection(self.sorted_formats,'PNG'))
    return get_template(req, 'downloadplot', context)
    
  def sendplot(self, req):
    """Send plot to user
    """
    plotid=req.get_form_var('plotid',None)
    format = req.get_form_var('format', 'PNG')
    print "sendplot", plotid, format
    req.response.set_content_type(self.supported_formats[format])
    #req.response.set_content_type('image/png')
    plot = self.datasource.get_plot_by_id(plotid)
    if format in self.vector_formats:
      # do funky thing involving writing to a file
      print "FIXME: support vector formats better"
      return plot.iplot.tostring(format)
    else:
      # just grab tostring
      return plot.iplot.tostring(format)
      
    
class LabelingSourceUI:
  _q_exports = ['edit']
  def __init__(self, datamanagerui, datasource, labelingsource):
    self.datamanagerui = datamanagerui
    self.datasource = datasource
    self.labelingsource = self.datasource.get_labelingsource_by_id(labelingsource)
    if self.labelingsource is None:
      raise TraversalError("couldn't find labeling source %s" % (labelingsource))


  def edit(self, req):
    name = req.get_form_var('name', None)
    source = req.get_form_var('source', None)
    isrow = req.get_form_var('isrow', None)
    url = req.get_form_var('url', None)
    description = req.get_form_var('description', None)
    
    # convert isrow radio button to proper type
    if isrow is not None:
      if isrow=="row":
        isrow = True
      elif isrow=="column":
        isrow = False
      else:
        raise QueryError("Invalid value for isrow")

    # update any values the user passed in
    if name is not None:
      self.labelingsource.name = name
    if source is not None:
      self.labelingsource.source = source
    if isrow is not None:
      self.labelingsource.isrow = isrow
    if url is not None:
      self.labelingsource.lookup_url = url
    if description is not None:
      self.labelingsource.description = description

    return self._q_index(req)

  def _q_index(self, req):
    context = get_context(req, self.datasource)
  
    context.addGlobal('sourcetype', 'text')
    if self.labelingsource.isannotation:
      context.addGlobal('editlabeling_title', 'Edit Annotation')
    else:
      context.addGlobal('editlabeling_title', 'Edit Clustering')
  
    # generate error if someone's giving us a bogus id
    if self.labelingsource is not None:
      context.addGlobal('sourcetype', 'text')
      context.addGlobal('labelingsource', self.labelingsource)
      context.addGlobal('editlabeling', True)
      if self.labelingsource.isrow:
        context.addGlobal('isrow', 'checked')
      else:
        context.addGlobal('iscolumn', 'checked')
    else:
      # new upload form
      context.addGlobal('sourcetype', 'file')

    return get_template(req, 'editlabelingsource', context)

class LabelingManager:
  """Class to help with lookup up and editing labeling sources
  """
  _q_exports = [ 'add_annotation', 'add_clustering' ]
  def __init__(self, datamanagerui, datasource, baseurl):
    self.datamanagerui = datamanagerui
    self.datasource = datasource
    self.baseurl = baseurl
    
  def _q_lookup(self, req, name):
    """If it's not something we know about perhaps its a labelname
    """
    # FIXME: perhaps we should actually have a name in the url?
    # FIXME: doing it this way means that labelnames cant be the same
    # FIXME: as plot names
    return LabelingSourceUI(self.datamanagerui, self.datasource, name)

  def add_annotation(self, req):
    """Create a new labeling and mark it as an annotation (and thus
    prevent it from showing up in the clustering filter pages)
    """
    return self.add(req, isannotation=True)
    
  def add_clustering(self, req):
    """Create a new labeling and mark it as a clustering and thus
    showing up in the clustering drop downs  but not the annotation drop
    downs
    """
    return self.add(req, isannotation=False)
    
  def add(self, req, isannotation):
    """Attach new labeling source
    """
    name = req.get_form_var('name', None)
    source = req.get_form_var('source', None)
    isrow = req.get_form_var('isrow', None)
    url = req.get_form_var('url', None)
    description = req.get_form_var('description', None)
    
    # convert isrow radio button to proper type
    if isrow is not None:
      if isrow=="row":
        isrow = True
      elif isrow=="column":
        isrow = False
      else:
        raise QueryError("Invalid value for isrow")

    if name is not None and source is not None and isrow is not None:
      # FIXME: session issue
      source = req.session.datamanager.save_temp_file(source.tmp_filename, source.base_filename, self.datasource)
      labelingsource = self.datasource.add_labeling(name, source, isrow, isannotation, url, description)
      return req.redirect(self.baseurl )
    else:
      context = get_context(req, self.datasource)
      if isannotation:
        context.addGlobal('editlabeling_title', 'Add Annotation')
      else:
        context.addGlobal('editlabeling_title', 'Add Clustering')
      context.addGlobal('editlabeling', False)
      context.addGlobal('sourcetype', 'file')
      return get_template(req, 'editlabelingsource', context)

  
class DatasourceUI:
  """Wrap a datasource object and give it a web interface
  """
  _q_exports = ['_q_index', 'edit', 'view', 'save', 'unload', 'delete', 'plots', 'labelplot', 'trajectory_plot', 'cluster_detail_plot', 'cluster_trajectories', 'confusion_matrix_plot', 'roc', 'labels', 'cluster', 'pca_projection', 'pca_eigenvector']
  
  def __init__(self, datamanagerui, datasource):
    self.datamanagerui = datamanagerui
    self.datasource = datasource
    self.baseurl = _path_join(ROOT_URL, 'compclust', self.datasource.id)
    self.ploturl = _path_join(self.baseurl, "plots")
    self.downloadploturl = _path_join(self.baseurl, "plots", "download")
    self.__labels = LabelingManager(self.datamanagerui, self.datasource, self.baseurl)
    self.cluster = ClusteringManagerUI(self.datamanagerui, self.datasource, self.baseurl)
    self.plots = PlotUI(self.datasource)

  def __get_labels(self):
    return self.__labels
  labels = property(__get_labels, doc="Return labelingmanager")

  def _q_index(self,req):
    context = get_context(req, self.datasource)
    return get_template(req, 'Datasource', context)

  
  def view(self, req):
    # render page
    context = get_context(req, self.datasource)
    context.addGlobal('datasourcename', self.datasource.name)
    if self.datasource.is_loaded:
      shape = (self.datasource.dataset.numRows,self.datasource.dataset.numCols)
      context.addGlobal('datasourcesize', "%s x %s"%(shape))
    else:
      context.addGlbal('datasourcesize', "not loaded")

    labelings = self.datasource.get_labeling_sources()
    addlabel_qs = os.path.join(self.baseurl, "labels/%s")
    editlabel_qs = os.path.join(self.baseurl, "labels/%s/")
    annotations = [ (editlabel_qs % (l.id), l) for l in labelings
                    if l.isannotation == True]
    clusterings = [ (editlabel_qs % (l.id), l) for l in labelings
                    if l.isannotation==False]
    context.addGlobal('annotations', annotations)
    context.addGlobal('clusterings', clusterings)
    context.addGlobal('datasourcesource', self.datasource.source)
    context.addGlobal('add_annotation_url', addlabel_qs % ('add_annotation'))
    context.addGlobal('add_clustering_url', addlabel_qs % ('add_clustering'))
    # attach variable describing available labelings and which is selected
    self.add_annotation_context(context,'primary',self.datasource.primary)
    self.add_annotation_context(context,'secondary',self.datasource.secondary)
    context.addGlobal('save_url', os.path.join(self.baseurl, 'save'))
    context.addGlobal('unload_url', os.path.join(self.baseurl, 'unload'))
    context.addGlobal('delete_url', os.path.join(self.baseurl, 'delete'))
    context.addGlobal('loaded', self.datasource.is_loaded)
    return get_template(req, 'Datasource_view', context)

  def edit(self, req):
    """Update settings for a data source
    """
    datasourcename = req.get_form_var('datasourcename', None)
    primary = req.get_form_var('primary', None)
    secondary = req.get_form_var('secondary', None)

    if datasourcename is not None:
      self.datasource.name = datasourcename
    if primary is not None and primary != "None":
      self.datasource.primary = primary
    if secondary is not None and secondary != "None":
      self.datasource.secondary = secondary
    return req.redirect(self.baseurl)
    # TODO: add the ability to change a datasource

  def save(self, req):
    """Save datasource to persistence cache
    """
    # FIXME: session issue
    req.session.datamanager.persist(self.datasource.id)
    
    html = [template_header()]
    html += [self.datamanagerui.menu(req, self.datasource)]
    html += ["<p>saved %s to persistent store</p>" % self.datasource.getShortenedName()]
    html += [template_footer()]
    return string.join(html, os.linesep)
  
  def unload(self, req):
    """Try to remove dataset from memory
    """
    # FIXME: session issue
    req.session.datamanager.unload(self.datasource.id)

    html = [template_header()]
    html += [self.datamanagerui.menu(req, self.datasource)]
    html += ["<p>unloaded %s from memory</p>" % self.datasource.getShortenedName()]
    html += [template_footer()]
    return string.join(html, os.linesep)

  def delete(self, req):
    """Delete datasource from persistence cache
    """
    # FIXME: session issue
    req.session.datamanager.delete(self.datasource.id)

    html = [template_header()]
    html += [self.datamanagerui.menu(req, self.datasource)]
    html += ["<p>Removed %s from persistent store</p>" % self.datasource.getShortenedName()]
    html += [template_footer()]
    return string.join(html, os.linesep)

  def labelplot(self, req):
    """Show plot for a particular trajectory including its labelings
    """
    if self.datasource.primary is None:
      raise PublishError(public_msg="Please go to the dataset edit page and set a primary labeling")
    
    subset = req.get_form_var('subset', None)
    set = req.get_form_var('set', None)
  
    context = get_context(req, self.datasource)
    self.add_clustering_context(context, 'set', set)
  
    # convert from string to actual labeling
    set = self.datasource.get_labeling_by_name(set)

    print "1 views: ", len(self.datasource.dataset.getViews())
    datasetplot = self.datasource.dataset_plot(subset_label=subset, subset=set)
    print "2 views: ", len(self.datasource.dataset.getViews())
  
    detail_fmt = "%s <Labeling %s: %d labels>"
    detail=[ detail_fmt %(label, labeling.getName(), len(labeling.getLabels()))
             for labeling, label in self.datasource.get_labelings_by_primary(subset)]
    
    context.addGlobal('labeling_detail', detail)
    context.addGlobal('labelplot', os.path.join(self.ploturl, datasetplot.id))
    context.addGlobal('downloadploturl',
                      self.downloadploturl+"?plotid="+str(datasetplot.id))
    context.addGlobal('csimap', datasetplot.get_data_imagemap(self.baseurl, set))
    return get_template(req, 'labelplot', context)

  def trajectory_plot(self, req):
    """EXPERIMENTAL Plot for working toward web accessible pca viewer
    """
    #subset_label = req.get_form_var('subset_label', None)
    #color_label = req.get_form_var('color_label', None)
    #color_label = 'pc01-outlier-probes'
    set = 'pc01-outlier-probes'
  
    context = get_context(req, self.datasource)
    # save combo settings
    #context.addGlobal('subset_label', subset_label)
    #self.add_clustering_context(context, 'set', set)
  
    # get labelings instead of simple strings
    set = self.datasource.get_labeling_by_name(set)
    datasetplot = self.datasource.dataset_plot()
    
    context.addGlobal('cluster_detail_plot', os.path.join(self.ploturl, datasetplot.id))
    context.addGlobal('downloadploturl',
                      self.downloadploturl+"?plotid="+str(datasetplot.id))
    context.addGlobal('csimap', datasetplot.get_data_imagemap(self.baseurl, self.datasource.primary))
    return get_template(req, 'cluster_detail_plot', context)
  
  def cluster_detail_plot(self, req):
    subset_label = req.get_form_var('subset_label', None)
    set = req.get_form_var('set', None)
  
    context = get_context(req, self.datasource)
    # save combo settings
    context.addGlobal('subset_label', subset_label)
    self.add_clustering_context(context, 'set', set)
  
    # get labelings instead of simple strings
    set = self.datasource.get_labeling_by_name(set)
  
    datasetplot = self.datasource.dataset_plot(subset_label=subset_label, subset=set)
    
    context.addGlobal('cluster_detail_plot', os.path.join(self.ploturl, datasetplot.id))
    context.addGlobal('downloadploturl',
                      self.downloadploturl+"?plotid="+str(datasetplot.id))
    context.addGlobal('csimap', datasetplot.get_data_imagemap(self.baseurl, self.datasource.primary))
    return get_template(req, 'cluster_detail_plot', context)

  def cluster_trajectories(self, req):
    set = req.get_form_var('set', None)
    context = get_context(req, self.datasource)
    self.add_clustering_context(context, 'set', set)
    
    if set is not None and set != "None":
      # we have enough information to create a plot
      set = self.datasource.get_labeling_by_name(set)
      ts_plot = self.datasource.cluster_trajectories(set)
      context.addGlobal('cluster_trajectories_plot',
                        os.path.join(self.ploturl,
                                     ts_plot.id))
      context.addGlobal('downloadploturl', 
                        self.downloadploturl+"?plotid="+str(ts_plot.id))
                                                        
      context.addGlobal('csimap', ts_plot.get_axes_imagemap(self.baseurl))
    return get_template(req, 'cluster_trajectories', context)
  
  def confusion_matrix_plot(self, req):
    labeling1 = req.get_form_var('labeling1', None)
    labeling2 = req.get_form_var('labeling2', None)
    
    # store our parameters
    context = get_context(req, self.datasource)
    self.add_clustering_context(context, 'labeling1', labeling1)
    self.add_clustering_context(context, 'labeling2', labeling2)
  
    # convert StringFields to labelings
    labeling1 = self.datasource.get_labeling_by_name(labeling1)
    labeling2 = self.datasource.get_labeling_by_name(labeling2)
      
    if labeling1 is not None and labeling2 is not None:
      # make plot
      cm_plot = self.datasource.confusion_matrix_summary(labeling1, labeling2)
      context.addGlobal('confusion_matrix_plot',
                        os.path.join(self.ploturl, cm_plot.id))
      context.addGlobal('downloadploturl',
                        self.downloadploturl+"?plotid="+str(cm_plot.id))
      context.addGlobal('csimap', cm_plot.get_axes_imagemap(self.baseurl))
  
    return get_template(req, 'confusion_matrix_plot', context)
  
  def roc(self, req):
    """Render a roc plot.
  
    label is the label element to compare to.
    labeling is which labeling the comparison is supposed to be done with
    distance is distance metric.
    """
    labeling = req.get_form_var('labeling', None)
    # since most of the code checks for the Python None type to avoid
    # processing we should change the text representation to the python
    # type
    if labeling == "None":
      labeling = None
      
    label = req.get_form_var('label', None)
    distance = req.get_form_var('distance', None)
    
    context = get_context(req, self.datasource)
    self.add_clustering_context(context, 'labeling', labeling)
  
    labelinge = self.datasource.get_labeling_by_name(labeling)

    if labeling is not None:
      # get labels from labeling
      labels = self.datasource.get_labels_for_labeling(labeling)
      context.addGlobal('labels', html_combo_selection(labels, label))
      if label is not None:
        if label not in self.datasource.get_labels_for_labeling(labeling):
          raise QueryError("unrecognized label for %s" % (labeling))
      context.addGlobal('prompt_for_label', True)
    else:
      context.addGlobal('prompt_for_label', False)
      
    if distance is None:
      distance = DistanceMetrics.EuclideanDistance
    
    if labeling is not None and label is not None:
      labeling = self.datasource.get_labeling_by_name(labeling)
      roc_plot = self.datasource.roc_plot(label, labeling, distance)
      context.addGlobal('rocplot', os.path.join(self.ploturl, roc_plot.id))
      context.addGlobal('downloadploturl',
                        self.downloadploturl+"?plotid="+str(roc_plot.id))
      
    return get_template(req, 'roc', context)

  def pca_projection(self, req):
    xaxis_label = req.get_form_var('xaxis', None)
    yaxis_label = req.get_form_var('yaxis', None)

    # figure out which axis to plot
    if xaxis_label is None:
      xaxis = 0
    else:
      xaxis = int(string.split(xaxis_label)[0])-1
      
    if yaxis_label is None:
      yaxis = 1
    else:
      yaxis = int(string.split(yaxis_label)[0])-1

    # make plot
    pca_plot =self.datasource.pca_projection(xaxis, yaxis)
    pca_iplot = pca_plot.iplot
    variances = pca_iplot.getEigenVariances()

    # make plot label
    if xaxis_label is None:
      xaxis_label = "%s - %.2f" % (1, variances[0]*100)
    if yaxis_label is None:
      yaxis_label = "%s - %.2f" % (2, variances[1]*100)

    xaxis_list= ["%s - %.2f"%(x+1, variances[x]*100) for x in xrange(len(variances))]
    xaxis_list= html_combo_selection(xaxis_list, xaxis_label)
    yaxis_list= ["%s - %.2f"%(y+1, variances[y]*100) for y in xrange(len(variances))]
    yaxis_list= html_combo_selection(yaxis_list, yaxis_label)

    # this actually forces the image to be saved
    pca_iplot.image_filename
    
    set = self.datasource.primary.name

    # store our parameters
    context = get_context(req, self.datasource)
    context.addGlobal('xaxes', xaxis_list)
    context.addGlobal('yaxes', yaxis_list)

    context.addGlobal('pcaplot', os.path.join(self.ploturl, pca_plot.id))
    context.addGlobal('downloadploturl',
                      self.downloadploturl+"?plotid="+str(pca_plot.id))
    context.addGlobal('csimap', pca_plot.get_scatter_imagemap(self.baseurl, set))
  
    return get_template(req, 'pca_projection', context)

  def pca_eigenvector(self, req):
    eigen_plot =self.datasource.pca_eigenvector()
    # store our parameters
    context = get_context(req, self.datasource)

    context.addGlobal('eigenplot', os.path.join(self.ploturl, eigen_plot.id))
    context.addGlobal('downloadploturl',
                      self.downloadploturl+"?plotid="+str(eigen_plot.id))
    return get_template(req, 'pca_eigen', context)
    

  #####
  # Utility functions for constructing web pages
  def add_annotation_context(self, context, name, value):
    """come up with values for combo boxes containing annotations
    value indicates which value is currently selected
    """
    return self.add_label_context(context, name, value, isannotation=True)

  def add_clustering_context(self, context, name, value):
    """come up with values combo boxes containing clusterings
    """
    return self.add_label_context(context, name, value, isannotation=False)
    #return self.add_all_label_context(context, name, value, isannotation=False)

  def add_label_context(self, context, name, value, isannotation):
    """come up with values combo boxes containing any type of labeling
    """
    labels = self.datasource.get_labelingsource_names()
    labels = [ l for l in self.datasource.get_labelingsource_names() if self.datasource._labeling_sources[l].isannotation == isannotation ]
    labels.insert(0, "None")
    if name == 'primary' and value is None:
      value = self.datasource.primary
    elif name == 'secondary' and value is None:
      value = self.datasource.secondary
    context.addGlobal(name, html_combo_selection(labels, value))

  def add_all_label_context(self, context, name, value, isannotation):
    """Get all the labelings attached to a dataset
    """
    labels = self.datasource.dataset.getLabelings()
    labels.extend([ l for l in self.datasource.get_labelingsource_names() if self.datasource._labeling_sources[l].isannotation == isannotation ])
    labels.sort()
    labels.insert(0, "None")
    if name == 'primary' and value is None:
      value = self.datasource.primary
    elif name == 'secondary' and value is None:
      value = self.datasource.secondary
    context.addGlobal(name, html_combo_selection(labels, value))

  def sort_labeling_by_name(self, a_labeling, b_labeling):
    a = str(a_labeling)
    b = str(b_labeling)
    if a < b:
      return -1
    elif a > b:
      return 1
    else:
      return 0

def html_combo_selection(choices, selection):
  """Create a list of containing 'selected' in which ever position
  of the choices list which equals 'selection'
  """
  # if we pass in a labelingsource get a meaningful string name
  if isinstance(selection, LabelingSource):
    selection = selection.name
  if selection is None:
    selection = "None"
  selected_flags = []
  for c in choices:
    if c == selection:
      selected_flags.append("selected")
    else:
      selected_flags.append(None)
  return zip(choices, selected_flags)
    
      
class DatamanagerUI:
  _q_exports = ['login', 'logout', 'plots', 'uploaddata', 'uploaddata_action']
  def __init__(self):
    self.loginurl = _path_join(ROOT_URL, 'compclust', 'login')
    
  def _q_access(self, req):
    if req.get_path() == self.loginurl:
      return
    elif req.session.user is None:
      raise NotLoggedInError("You must be logged in", req.get_url())

  def _q_exception_handler(self, req, exception):
    if isinstance(exception, NotLoggedInError):
      context = simpleTALES.Context()
      # save the url the user tried to connect to
      context.addGlobal('loginurl', self.loginurl)
      context.addGlobal('source_url', exception.source_url)
      return get_template(req, 'login', context)
    else:
      return default_exception_handler(req, exception)

  def login(self, req):
    username = req.get_form_var('username', None)
    password = req.get_form_var('password', None)
    source_url = req.get_form_var('source_url', None)
    if req.session.initialize(username):
      if source_url is not None and len(source_url) > 0:
        return req.redirect(source_url)
      else:
        return req.redirect(ROOT_URL, 'compclust')
    else:
      context = simpleTALES.Context()
      context.addGlobal('source_url', source_url)
      context.addGlobal('loginurl', self.loginurl)
      return get_template(req, 'login')
    
  def logout(self, req):
    get_session_manager().expire_session(req)
    return get_template(req, 'logout')

  def menu(self, req, datasource=None):
    datasource_head ='<li><a href="%(rooturl)s%(dsid)s">DS: %(dsname)s</a></li>'
    view_head ='<li><a href="%(rooturl)s%(dsid)s"> V: %(dsname)s</a></li>'
    analysis_detail = """\
    <ul>
      <li><a href="%(rooturl)s/%(dsid)s/view">edit</a></li>
      <li><a href="%(rooturl)s/%(dsid)s/cluster">clustering</a></li>
      <li><a href="%(rooturl)s/%(dsid)s/cluster_trajectories">cluster trajectories</a></li>
      <li><a href="%(rooturl)s/%(dsid)s/confusion_matrix_plot">confusion matrix</a></li>
      <li><a href="%(rooturl)s/%(dsid)s/roc">roc analysis</a></li>
      <li><a href="%(rooturl)s/%(dsid)s/pca_projection">PCA Projection</a></li>
      <li><a href="%(rooturl)s/%(dsid)s/pca_eigenvector">PCA Eigen Vectors</a></li>
    </ul>
"""

    analysis_list = []
    # FIXME: session issue
    for dsid, dsname, dslabelings, isloaded in req.session.datamanager.browse_list():
      if isloaded:
        params = {"rooturl": _path_join(ROOT_URL, 'compclust/'), "dsid": dsid, "dsname": dsname}
        
        analysis_list.append(datasource_head % (params))
        if datasource is not None and datasource.id == dsid:
          analysis_list.append(analysis_detail % (params))
          for view in datasource.dataset.getViews():
            params['dsname'] = str(view)
            analysis_list.append(view_head % (params))
    
    mainmenu = """\
  <div id="compclustMenu">
  <h1>Navigation Menu</h1>
  <ul class="linkMenu">
    <li><a href="%(rooturl)s/logout">Logout</a></li>
    <hr/>
    <li><a href="%(rooturl)s">Available Datasets</a></li>
    <li><a href="%(rooturl)s/uploaddata">Upload Dataset</a></li>
    <hr/>
    %(analysis)s
  </ul>
  </div>
""" % {'rooturl': _path_join(ROOT_URL, 'compclust'),
       'analysis': string.join(analysis_list, os.linesep)}

    return mainmenu

  def _q_lookup(self, req, name):
    try:
      # FIXME: session issue
      datasource = req.session.datamanager[name]
    except KeyError, e:
      raise TraversalError("unrecognized datasource id")
    return DatasourceUI(self, datasource)
      
  def _q_index(self, req):
    """Show list of datasources
    """
    context = get_context(req)
    browse = [{'dsid': ds[0],
               'dsname': ds[1],
               'labelingcount': len(ds[2]),
               'edit': _path_join(ROOT_URL, 'compclust', ds[0], ''),
               'save': _path_join(ROOT_URL, 'compclust', ds[0], '/save'),
               'unload':_path_join(ROOT_URL, 'compclust', ds[0], '/unload'),
               'delete':_path_join(ROOT_URL, 'compclust', ds[0], '/delete'),
               'loaded': ds[3],
               'notloaded': not ds[3]}
               # FIXME: session issue
               for ds in req.session.datamanager.browse_list()]
    context.addGlobal('ds_browse', browse)
    return  get_template(req, 'index', context)

  def uploaddata(self, req):
    context = get_context(req)
    return get_template(req, 'uploaddata', context)
  
  def uploaddata_action(self, req):
    datasourcename = req.get_form_var('datasourcename', None)
    datasource = req.get_form_var('datasource', None)
    datasourceurl = req.get_form_var('datasourceurl', None)
    
    # add new datasource
    if datasource is not None:
      # we have a datasource to add.
      ds = DataSource(datasource.tmp_filename)
    elif datasourceurl is not None:
      ds = DataSource(str(datasourceurl))
    else:
      ds = None
  
    if ds is None:
      raise PublishError("Unable to load data")
    
    if datasourcename is not None:
      ds.name = str(datasourcename)
    # FIXME: session issue
    req.session.datamanager.append(ds)

    return req.redirect(_path_join(ROOT_URL, 'compclust', ds.id))


compclust = DatamanagerUI()

###############
# "Pages" to return
def _q_index(req):
  return compclust._q_index(req)
  context = get_context(req)
  return get_template(req, 'intro', context)

