#####
# License: unknown probably Caltech

import mimetypes
import os
import posixpath
import string
import StringIO
import sys
import urllib


class Request(object):
  """Simple simulation of the mod_python request object
  """
  def __init__(self, baseRequestHandler):
    self.baseRequestHandler = baseRequestHandler
    self.headers_in = baseRequestHandler.headers
    self.uri, self.args = _split_query(baseRequestHandler.path)
    self.filename = _translate_path(self.uri)

    self.content_type = mimetypes.guess_type(self.filename)
    self.__document = StringIO.StringIO()

    self.method = baseRequestHandler.command
    
  def sendfile(self, filename):
    f = open(filename, 'r')
    buf = f.read()
    while len(buf) > 0:
      self.write(buf)
      buf = f.read()

  ######
  # Magic request object
  # reads are directored toward the data sent by the browser
  # writes are directed toward what we're sending back to the browser

  # read data
  def getClosed(self):
    return self.baseRequestHandler.rfile.closed
  closed = property(getClosed, None, None, doc="is read buffer closed?")
    
  def read(self, len=0):
    return self.baseRequestHandler.rfile.read(len)

  def readline(self, size=-1):
    return self.baseRequestHandler.rfile.readline(size)
  
  def readlines(self):
    return self.baseRequestHandler.rfile.readlines(len)


  # write data
  def getLen(self):
    return self.__document.len
  len = property(getLen, doc="length of the return document")
  
  def getvalue(self):
    """return contents of generated document
    """
    return self.__document.getvalue()

  def write(self, s):
    self.__document.write(s)

class Table(dict):
  def add(self, key, value):
    self[key] = value

def make_table():
  return Table()

def _translate_path(path):
  """Translate a /-separated PATH to the local filename syntax.

  Components that mean special things to the local file system
  (e.g. drive or directory names) are ignored.  (XXX They should
  probably be diagnosed.)

  """
  path = posixpath.normpath(urllib.unquote(path))
  words = path.split('/')
  words = filter(None, words)
  path = os.getcwd()
  for word in words:
      drive, word = os.path.splitdrive(word)
      head, word = os.path.split(word)
      if word in (os.curdir, os.pardir): continue
      path = os.path.join(path, word)
  return path

def _split_query(path):
  """Split off the query string
  """
  split = string.split(path, '?', 1)
  if len(split) == 1:
    return (split[0], '')
  else:
    return split
  
    
###############
# Useful constants

HTTP_CONTINUE                     = 100
HTTP_SWITCHING_PROTOCOLS          = 101
HTTP_PROCESSING                   = 102
HTTP_OK                           = 200
HTTP_CREATED                      = 201
HTTP_ACCEPTED                     = 202
HTTP_NON_AUTHORITATIVE            = 203
HTTP_NO_CONTENT                   = 204
HTTP_RESET_CONTENT                = 205
HTTP_PARTIAL_CONTENT              = 206
HTTP_MULTI_STATUS                 = 207
HTTP_MULTIPLE_CHOICES             = 300
HTTP_MOVED_PERMANENTLY            = 301
HTTP_MOVED_TEMPORARILY            = 302
HTTP_SEE_OTHER                    = 303
HTTP_NOT_MODIFIED                 = 304
HTTP_USE_PROXY                    = 305
HTTP_TEMPORARY_REDIRECT           = 307
HTTP_BAD_REQUEST                  = 400
HTTP_UNAUTHORIZED                 = 401
HTTP_PAYMENT_REQUIRED             = 402
HTTP_FORBIDDEN                    = 403
HTTP_NOT_FOUND                    = 404
HTTP_METHOD_NOT_ALLOWED           = 405
HTTP_NOT_ACCEPTABLE               = 406
HTTP_PROXY_AUTHENTICATION_REQUIRED= 407
HTTP_REQUEST_TIME_OUT             = 408
HTTP_CONFLICT                     = 409
HTTP_GONE                         = 410
HTTP_LENGTH_REQUIRED              = 411
HTTP_PRECONDITION_FAILED          = 412
HTTP_REQUEST_ENTITY_TOO_LARGE     = 413
HTTP_REQUEST_URI_TOO_LARGE        = 414
HTTP_UNSUPPORTED_MEDIA_TYPE       = 415
HTTP_RANGE_NOT_SATISFIABLE        = 416
HTTP_EXPECTATION_FAILED           = 417
HTTP_UNPROCESSABLE_ENTITY         = 422
HTTP_LOCKED                       = 423
HTTP_FAILED_DEPENDENCY            = 424
HTTP_INTERNAL_SERVER_ERROR        = 500
HTTP_NOT_IMPLEMENTED              = 501
HTTP_BAD_GATEWAY                  = 502
HTTP_SERVICE_UNAVAILABLE          = 503
HTTP_GATEWAY_TIME_OUT             = 504
HTTP_VERSION_NOT_SUPPORTED        = 505
HTTP_VARIANT_ALSO_VARIES          = 506
HTTP_INSUFFICIENT_STORAGE         = 507
HTTP_NOT_EXTENDED                 = 510

OK = HTTP_OK
REQ_PROCEED = 0
DONE = -2
DECLINED = REQ_NOACTION = -1
