#!/usr/bin/python
import os
import unittest


from compClust.mlx import datasets
from compClust.mlx import labelings

from compClust.iplot.views import DatasetRowPlotView
from compClust.util.LoadExample import LoadCho

class testIPlot(unittest.TestCase):
  def setUp(self):
    self.ds, labelings_dictionary = LoadCho()
    for k, v in labelings_dictionary.items():
      setattr(self, k, v)
    
    
  def testSimplePlot(self):
    # plot some simple data
    dp = self.dataset_plot()
    v = DatasetRowPlotView(self.ds,
                           primaryLabeling=self.commonName,
                           secondaryLabeling=self.choCluster)
    dp.plot(v)

  def testTrajectorySummaryPlot(self):
    cho_cluster = self.ds.getLabeling("Cho")
    names = self.ds.getLabeling("common name")

    self.trajectory_summary_plot(self.ds, cho_cluster, primaryLabeling=names)
    #TrajectorySummary(ds, classification, primaryLabeling=primary)

  def testConfusionMatrixSummaryPlot(self):
    cho_cluster = self.ds.getLabeling("Cho")
    diagem_cluster = self.ds.getLabeling("diagem")
    names = self.ds.getLabeling("common name")
    self.confusion_matrix_plot(self.ds, cho_cluster, diagem_cluster, names)
    
    #self.confusion_matrix_plot(self.ds, self.choCluster, self.diagemCluster, self.commonName)

  def testROCPlot(self):
    cho_cluster = self.ds.getLabeling("Cho")
    diagem_cluster = self.ds.getLabeling("diagem")

    self.roc_plot(self.ds, cho_cluster, 'Early G1')
    #self.roc_plot(self.ds, diagem_cluster, '3')
    
  def testPCAPlot(self):
    orfs = self.ds.getLabeling("orfs")
    names = self.ds.getLabeling("common name")
    pcaplot = self.pca_plot(self.ds, orfs, names)
    pcaplot.ProjectionPlot(2,3)

  def testPCAGinsu(self):
    orfs = self.ds.getLabeling("orfs")
    names = self.ds.getLabeling("common name")
    times = self.ds.getLabeling("Time Points")
    self.ds.primaryRowLabeling = orfs
    self.ds.primaryColumnLabeling = times
    ginsu = self.pca_ginsu(self.ds, secondaryLabeling=names)
    ginsu.plotPCNOutlierRowsInOriginalColumnOrder(1)
    ginsu.plotPCNOutlierRowsInSigGroupOrder(1)
    ginsu.plotPCvsPCWithOutliersInY(0,1)
    self.ds.primaryRowLabeling = None
    self.ds.primaryColumnLabeling = None
