import os
import tempfile
import unittest

from compClust.iplot.CanvasFactoryAgg import CanvasFactoryAgg
import compClust.iplot.Plot as Plot

class testPlot(unittest.TestCase):
  def testInitializedWithCanvas(self):
    """Test things that should be true if we initialized a plot with
    a canvas factory
    """
    canvas = CanvasFactoryAgg()
    p = Plot.Plot(canvas)

    self.failUnless(canvas == p.getCanvasFactory())
    self.failUnlessRaises(ValueError, p.setCanvasFactory, self)
    p.setCanvasFactory(None)
    self.failUnless(hasattr(p, "canvas"), "we should have a canvas")

  def testInitilizeWithoutCanvas(self):
    """Test things that should be true if we were created without a
    canvas factory
    """
    p = Plot.Plot()
    canvas = CanvasFactoryAgg()

    self.failUnless(p.canvas == None , "there should be no canvas")
    self.failUnless(p.getCanvasFactory() == None)
    p.setCanvasFactory(canvas)
    self.failUnless(hasattr(p, "canvas"),
                    "after setting the factory we should have a canvas")

  def testSavePlot(self):
    p = Plot.Plot(CanvasFactoryAgg())
    fd, filename = tempfile.mkstemp(prefix="test")
    p.save( filename )
    
    
if __name__ == "__main__":
  suite = unittest.TestSuite()
  suite.addTest(unittest.makeSuite(testPlot))
  unittest.TextTestRunner(verbosity=2).run(suite)
