#!/usr/bin/env python2.2
########################################
# The contents of this file are subject to the MLX PUBLIC LICENSE version
# 1.0 (the "License"); you may not use this file except in
# compliance with the License.
# 
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See
# the License for the specific language governing rights and limitations
# under the License.
# 
# The Original Source Code is "compClust", released 2003 September 03.
# 
# The Original Source Code was developed by the California Institute of
# Technology (Caltech).  Portions created by Caltech are Copyright (C)
# 2002-2003 California Institute of Technology. All Rights Reserved.
########################################

"""
Test suite for the old Dataset schema (obsolete).
"""

import unittest
import os

from compClust.mlx.datasets import Dataset
import compClust.mlx

dataset_test_home    = "."
test_data_files_home = dataset_test_home + "/test"

dataset_spurious_filename = "dataset_tab_array_spurious_whitespace.txt"

def read_testset(filename):
  return Dataset(test_data_files_home + "/" + filename)
  
class DatasetTestCases(unittest.TestCase):
  def setUp(self):
    self.original_dir    = os.getcwd()
    os.chdir(compClust.mlx.__path__[0])

  def tearDown(self):
    os.chdir(self.original_dir)

  def check_other_subsets(self):
    """Test out more sophisticated subset operations"""

    dataset = read_testset(dataset_spurious_filename)

    #
    # Test partitioning into many subsets
    #

    labels   = map(str, range(26))
    labeling = Labeling.Labeling(labels)
    subsets = dataset.subsets(labeling)

    assert len(subsets) == 26
    for i in subsets.keys():
      assert subsets[i].getRowIDs()[0] not in labels
      
    labels   = ['1']*26
    labeling = Labeling.Labeling(labels)
    subsets = dataset.subsets(labeling)

    assert len(subsets) == 1
    assert subsets['1'].getRowIDs() == dataset.getRowIDs()

    labels   = ['0']*10 + ['1']*10 + ['2']*6
    labeling = Labeling.Labeling(labels)
    subsets = dataset.subsets(labeling)

    assert len(subsets) == 3
    for i in subsets.keys():
      for j in subsets[i].getRowIDs():
        assert j not in ['0', '1', '2']
        
    subset  = dataset.subset('0',labeling)
    assert subset is not None
    assert subset.getNumRows() == 10
    data1   = subset.getData()
    data2   = dataset.getData()
    for i in range(10):
      assert data1[i] == data2[i]
    

  def check_read_array_spurious_whitespace(self):
    """Try parsing an array with trailing delimiters.

    Read in an tab delimited array that has extra
    delimiters at the end and has scattered extra spaces
    in the file.
    """

    dataset = read_testset(dataset_spurious_filename)
    assert dataset.getNumRows() == 26
    assert dataset.getNumCols() == 4
    
    for row in dataset.getData():
      assert len(row) == 4

  def check_write_dataset(self):
    """Verify that we can write out a dataset correctly.
    """

    output_filename = "/dataset_tab_array_spurious_whitespace.out"

    # erase previous copy
    if os.access(test_data_files_home + output_filename, os.F_OK):
      os.remove(test_data_files_home +output_filename)

    # initialize our example dataset
    dataset = read_testset(dataset_spurious_filename)

    # create temp file
    output_stream = open(test_data_files_home +output_filename, "w")
    dataset.writeDataset(output_stream)
    output_stream.close()

    file_arguments  = test_data_files_home
    file_arguments += "/"
    file_arguments += dataset_spurious_filename
    file_arguments += " "
    file_arguments += test_data_files_home
    file_arguments += "/"
    file_arguments += output_filename
    
    #status = os.system("diff -q " + file_arguments + " > /dev/null")
    #if os.WEXITSTATUS(status) == 0:
    #  fail("write_data preserved meaningless whitespace.")

    #status = os.system("diff -w -q " + file_arguments + " > /dev/null")
    #if os.WEXITSTATUS(status) != 0:
    #  fail("write_data didn't preserve non-whitespace correctly.")
    
def suite():
  suite = unittest.TestSuite()
  #suite.addTest(DatasetTestCases("check_read_array_spurious_whitespace"))
  #suite.addTest(DatasetTestCases("check_write_dataset"))
  #suite.addTest(DatasetTestCases("check_basic_subsets"))
  #suite.addTest(DatasetTestCases("check_other_subsets"))
  return suite

if __name__ == "__main__":
  unittest.main(defaultTest="suite")
