########################################
# The contents of this file are subject to the MLX PUBLIC LICENSE version
# 1.0 (the "License"); you may not use this file except in
# compliance with the License.
# 
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See
# the License for the specific language governing rights and limitations
# under the License.
# 
# The Original Source Code is "compClust", released 2003 September 03.
# 
# The Original Source Code was developed by the California Institute of
# Technology (Caltech).  Portions created by Caltech are Copyright (C)
# 2002-2003 California Institute of Technology. All Rights Reserved.
########################################
#
#  Written By    :  Lucas Scharenbroich
#  Date          :  Oct 2001
#  
#  Description: Wraps the wmatch and wmatch-full commands 
#
#  Requires   : the environment variables WMATCH_COMMAND and
#               WMATCH_FULL_COMMAND be set.
#               (e.g. export WMATCH_COMMAND=/proj/code/bin/wmatch)
#
#################

import os
import re
import tempfile
import string
import Numeric

from compClust.util import Verify

def wmatch(matrix, full=0):

  """
  result = wmatch(matrix, full=0)

  Wraps the wmatch command.  If full is left as it's default, then result
  is the linear assignment score of the matrix returned as a float.

  If full is true, then result contains the full adjacency matrix returned as
  a Numeric array.
  """


  #
  # Verify that wmatch and wmatch-full are both defined
  #

  envNames = ["WMATCH_COMMAND",
              "WMATCH_FULL_COMMAND"]

  if Verify.environment_variables_exist( envNames ):
    return None

  #
  # Matrix OK, so now create the input file
  #

  conf_matrix_filename = tempfile.mktemp('wmatch-tmpfile')
  conf_matrix_file     = open(conf_matrix_filename, "w")

  #
  # write out confusion matrix header
  #

  rows = len(matrix)
  cols = len(matrix[0])
  
  conf_matrix_file.write("%d %d\n" % (rows, cols))

  #
  # write out body
  #
  
  for r in range( rows ):
    for c in range( cols ):
      conf_matrix_file.write(str(matrix[r][c])+" ")
    conf_matrix_file.write("\n")

  #
  # close the file
  #
  
  conf_matrix_file.close()

  #
  # run the command
  #

  if full:
    wmatch_cmd = os.environ["WMATCH_FULL_COMMAND"]
  else:
    wmatch_cmd = os.environ["WMATCH_COMMAND"]
    
  try:
    result_child_out = os.popen(wmatch_cmd + " " + conf_matrix_filename, "r")
  except:
    sys.exit("WMATCH FAILED TO RETURN")
            
  result_lines = result_child_out.readlines()
  #result_child_out.close()

  if len(result_lines) == 0:
    raise OSError("No output from wmatch: " + str(result_lines))

      
  #
  # Remove the temp input file
  #
  os.remove(conf_matrix_filename)
  
  if full:

    result = Numeric.zeros((rows, cols))

    #i = 0
    #count = 0
    #while count < rows:
    for i in range(0, rows):
      x, y = string.split(result_lines[i])
      x    = int (x)
      y    = int (y)
      i+=1
      if  x == 0 or y == 0:
        continue
      if x > rows:
        x = x - rows
      if y > rows:
        y = y - rows
      
      result[x-1, y-1] = 1
      #count +=1
  else:

    #
    # parse output of command
    #

    result_fields = re.split(" ", result_lines[0])
    if len(result_fields) != 2:
      raise OSError("Unknown result from wmatch: " + str(result_lines))

    #
    # second field contains the numeric score
    #

    result =  float(result_fields[1])

  return result
