#!/usr/bin/env python
########################################
# The contents of this file are subject to the MLX PUBLIC LICENSE version
# 1.0 (the "License"); you may not use this file except in
# compliance with the License.
# 
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See
# the License for the specific language governing rights and limitations
# under the License.
# 
# The Original Source Code is "compClust", released 2003 September 03.
# 
# The Original Source Code was developed by the California Institute of
# Technology (Caltech).  Portions created by Caltech are Copyright (C)
# 2002-2003 California Institute of Technology. All Rights Reserved.
########################################

"""Setup script for compClust

The one interesting detail is the command line takes a parameter --notest
to turn off the unit tests.
"""
from distutils.core import setup
from distutils.core import Extension
try:
  pyrex = True
  from Pyrex.Distutils import build_ext
except ImportError:
  print "Please install pyrex <http://pyrex.sf.net>"
  pyrex = False

# default to running tests
do_tests = True
# don't default to symlinking pyrex extensions into the source tree
# (which is convienent when doing development)
inplace = False
  
from getopt import getopt
import glob
import os
import re
import string
import shutil
import sys

########################################
# Setup Configuration
PACKAGES=["compClust",
          "compClust.gui",
          "compClust.iplot",
          "compClust.mlx",
          "compClust.mlx.DA",
          "compClust.mlx.datasets",
          "compClust.mlx.labelings",
          "compClust.mlx.models",
          "compClust.mlx.views",
          "compClust.mlx.wrapper",
          "compClust.score",
          "compClust.util",
# to beta to deal with
#          "compClust.visualize",
#          "compClust.visualize.dataViewer",
#          "bioinformatics",
#          "bioinformatics.schema",
#          "bioinformatics.schema.datasets",
#          "bioinformatics.schema.views",
#          "bioinformatics.util",
#          "bioinformatics.visualize",          
          ]
def findScripts(startDir=None):
  if startDir is None:
    startDir = os.getcwd()
  else:
    startDir = os.path.realpath(startDir)  
  #
  scripts = []
  for dirEntry in filter(lambda x: not re.search("~$", x), os.listdir(startDir)):
    pathname = os.path.join(startDir, dirEntry)
    if re.search("(CVS$)|(.svn$)", pathname):
      continue
    elif os.path.isdir(pathname):
      # recurse through other directories
      scripts.extend(findScripts(pathname))
    elif os.path.isfile(pathname):
      # find files that are scripts
      f = open(pathname, 'r')
      if re.search('#!.*python', f.readline()):
        scripts.append(pathname)
      f.close()
  return scripts

ext_modules = []
#modules.append( Extension("cDAmodule", ["compClust/mlx/DA/DAmodule.c"]))
SCRIPTS = filter(lambda x: not re.search("/Test[^/.]*.py$", x),
                 findScripts('compClust'))

SCRIPTS.append(os.path.abspath('./compClust/gui/CompClustTk.py'))

if sys.platform == 'linux2':
  docPath = '/usr/share/doc/compClust'
elif sys.platform == 'win32':
  docPath = '..\\Program Files\\compClust\\docs'
else:
  docPath = 'compClust'

def vc_filter(x):
  """Filter for skipping version control subdirectories
  """
  if x.find('CVS') or x.find('.svn'):
    return 0
  return 1

def run_tests(inplace=False):
  # specify the exact location of the cho data
  # the testIPlot script looks for this environment variable
  # for grabbing the data location
  os.environ['CHO_DATA'] = os.path.join(os.getcwd(), 'compClust', 'gui', 'Examples', 'ChoCellCycling')

  testscript = 'testEverything.py'

  if not inplace:
    curdir = os.getcwd()
    for builddir in glob.glob('build/lib.*'):
      build_testscript = os.path.join(builddir, testscript)
      shutil.copy(testscript, build_testscript)
      os.chdir(builddir)
      os.system(string.join([sys.executable, testscript], " "))
      os.chdir(curdir)
  else:
    os.system(string.join([sys.executable, testscript], " "))

def setup_inplace():
  """Link shared libraries to source tree
  """
  builddirs = glob.glob('build/lib.*')
  if len(builddirs) > 1:
    print "can't run in place"
    return
  
  basedir = os.getcwd()
  builddir = os.path.join(basedir, builddirs[0])
  os.chdir(builddir)
  
  for dirpath, dirnames, filenames in os.walk('.'):
    for filename in filenames:
      if re.search("\.so$", filename):
        src_file = os.path.normpath(os.path.join(builddir, dirpath, filename))
        dest_file = os.path.normpath(os.path.join(basedir, dirpath, filename))
        if not os.path.islink(dest_file):
          os.symlink(src_file, dest_file)
  os.chdir(basedir)

#CompClustTk Tutorial
tutPath = os.path.join(docPath, 'compclusttk_tutorial')
TUTORIAL_TUPLE=(tutPath,
                filter(vc_filter,
                       glob.glob('compClust/gui/Docs/compclusttk_tutorial/*')))

#CompClustTk Example Data
examplePath = os.path.join(docPath, 'Examples', 'ChoCellCycling')
EXAMPLE_DATA_TUPLE=(examplePath,
                    filter(vc_filter,
                          glob.glob('compClust/gui/Examples/ChoCellCycling/*')))

#CompClustTk Data
DATA_FILES=[EXAMPLE_DATA_TUPLE, TUTORIAL_TUPLE]

# Pyrex extensions
if pyrex:
  graphmatching_dir = os.path.join(os.getcwd(), "compClust/mlx/graphmatching")
  graphmatching_modules = [ os.path.join(graphmatching_dir, x) for x in
                            ["graphmatching.pyx", "wmatch.c", "glib.c"]]
  graphmatching_package = "compClust.mlx.graphmatching"
  ext_modules.append(Extension(graphmatching_package, graphmatching_modules ))

ext_modules.append(Extension("compClust.mlx.DA.cDA", [os.path.join(os.getcwd(), "compClust/mlx/DA/DAmodule.c")]))

      
# Allow user to override testingg::
print sys.argv
for i in xrange(len(sys.argv)-1,-1,-1):
  if re.match("--?notest",sys.argv[i]):
    do_tests = False
    del sys.argv[i]
  elif re.match("--?inplace", sys.argv[i]):
    del sys.argv[i]
    inplace = True
    PACKAGES=None
    SCRIPTS=None
    DATA_FILES=None

d= setup(name="compClust",
      version="0.2",
      description="Comp Clust is a suite of tools for clustering  " \
      "and data mining. ",
      maintainer="Diane Trout",
      maintainer_email="diane@caltech.edu",
      url="http://woldlab.caltech.edu/compClust/",
      packages=PACKAGES,
      scripts=SCRIPTS,
      ext_modules=ext_modules,
      data_files=DATA_FILES,
      cmdclass = {'build_ext': build_ext})

if inplace: setup_inplace()
if do_tests: run_tests(inplace)
